@@ -20,7 +20,7 @@ import tarfile, os
 __copyright__ = (u"Copyright (c), 2017, Forschungszentrum Jülich GmbH, "
                  "IAS-1/PGI-1, Germany. All rights reserved.")
 __license__ = "MIT license, see LICENSE.txt file"
-__version__ = "0.7.0"
+__version__ = "0.7.7"
 __contributors__ = (u"Fabian Bertoldo", u"Philipp Ruessmann")
 
 #TODO: work on return results function
@@ -116,6 +116,7 @@ class kkr_imp_sub_wc(WorkChain):
         spec.input("kkrimp", valid_type=Code, required=True)
         spec.input("host_imp_startpot", valid_type=SinglefileData, required=False)
         spec.input("remote_data", valid_type=RemoteData, required=False)
+        spec.input("remote_data_Efshift", valid_type=RemoteData, required=False)
         spec.input("kkrimp_remote", valid_type=RemoteData, required=False)
         spec.input("impurity_info", valid_type=Dict, required=False)
         spec.input("options", valid_type=Dict, required=False,
@@ -172,7 +173,7 @@ class kkr_imp_sub_wc(WorkChain):
 
         # Define the outputs of the workflow
         spec.output('workflow_info', valid_type=Dict)
-        spec.output('host_imp_pot', valid_type=SinglefileData)
+        spec.output('host_imp_pot', valid_type=SinglefileData, required=False)
 
 
     def start(self):
@@ -199,6 +200,8 @@ class kkr_imp_sub_wc(WorkChain):
         self.ctx.last_remote = None
         # link to previous host impurity potential
         self.ctx.last_pot = None
+        # intermediate single file data objects that contain potential files which can be clean up at the end
+        self.ctx.sfd_pot_to_clean = []
         # convergence info about rms etc. (used to determine convergence behavior)
         self.ctx.last_rms_all = []
         self.ctx.rms_all_steps = []
@@ -374,6 +377,7 @@ class kkr_imp_sub_wc(WorkChain):
 
         # check if previous calculation was successful
         if self.ctx.loop_count>1 and not self.ctx.last_calc.is_finished_ok:
+            self.report("ERROR: last calc not finished_ok")
             return self.exit_codes.ERROR_SUB_FAILURE
 
         # next check only needed if another iteration should be done after validating convergence etc. (previous checks)
@@ -418,29 +422,28 @@ class kkr_imp_sub_wc(WorkChain):
                 decrease_mixing_fac = True
                 self.report("INFO: Last KKR did not converge. Trying decreasing mixfac")
                 # reset last_remote to last successful calculation
-                last_calcs_list = range(len(self.ctx.calcs))
+                last_calcs_list = list(range(len(self.ctx.calcs))) # needs to be list to support slicing
                 if len(last_calcs_list)>1: last_calcs_list = array(last_calcs_list)[::-1] # make sure to go from latest calculation backwards
                 for icalc in last_calcs_list:
                     self.report("INFO: last calc success? {} {}".format(icalc, self.ctx.KKR_steps_stats['success'][icalc]))
                     if self.ctx.KKR_steps_stats['success'][icalc]:
                         if self.ctx.KKR_steps_stats['last_rms'][icalc] < self.ctx.KKR_steps_stats['first_rms'][icalc]:
                             self.ctx.last_remote = self.ctx.calcs[icalc].outputs.remote_folder
+                            break # exit loop if last_remote was found successfully
                         else:
                             self.ctx.last_remote = None
-                        break # exit loop if last_remote was found successfully
                     else:
                         self.ctx.last_remote = None
-                self.report("INFO: Last_remote is None? {} {}".format(self.ctx.last_remote is None, 'structure' in self.inputs))
+                # now cover case when last_remote needs to be set to initial remote folder (from input)
                 if self.ctx.last_remote is None:
                     if 'kkrimp_remote' in self.inputs:
-                        self.ctx.last_remote = self.inputs.kkrimp_remote
                         self.report('INFO: no successful and converging calculation to take RemoteData from. Reuse RemoteData from input instead.')
+                        self.ctx.last_remote = self.inputs.kkrimp_remote
                     elif 'impurity_info' in self.inputs or 'remote_data' in self.inputs:
                         self.ctx.last_remote = None
                 # check if last_remote has finally been set and abort if this is not the case
-                self.report("INFO: last_remote is still None? {}".format(self.ctx.last_remote is None))
-                self.report("INFO: restart next calculation run from initial inputs")
                 if self.ctx.last_remote is None:
+                    self.report("ERROR: last remote not found")
                     return self.exit_codes.ERROR_SETTING_LAST_REMOTE
 
             # check if mixing strategy should be changed
@@ -448,8 +451,6 @@ class kkr_imp_sub_wc(WorkChain):
             if last_mixing_scheme is None:
                 last_mixing_scheme = 0
 
-            # TODO: problem with convergence on track has to be solved, just set as true for testing
-            #convergence_on_track = True
             if convergence_on_track:
                 last_rms = self.ctx.last_rms_all[-1]
                 if last_rms < self.ctx.threshold_aggressive_mixing and last_mixing_scheme == 0:
@@ -505,6 +506,7 @@ class kkr_imp_sub_wc(WorkChain):
                         new_params[key_default] = kkrdefaults.get(key_default)
                         kkrdefaults_updated.append(key_default)
                 if len(kkrdefaults_updated)>0:
+                    self.report("ERROR: no default param found")
                     return self.exit_codes.ERROR_MISSING_PARAMS
                 else:
                     self.report('updated KKR parameter node with default values: {}'.format(kkrdefaults_updated))
@@ -625,6 +627,7 @@ class kkr_imp_sub_wc(WorkChain):
                 for key, val in new_params.items():
                     para_check.set_value(key, val, silent=True)
             except:
+                self.report("ERROR: failed to set some parameters")
                 return self.exit_codes.ERROR_PARAMETER_UPDATE
 
             # step 3:
@@ -660,6 +663,10 @@ class kkr_imp_sub_wc(WorkChain):
         host_GF = self.inputs.remote_data
         imp_pot = self.ctx.last_pot
         last_remote = self.ctx.last_remote
+        if 'remote_data_Efshift' in self.inputs:
+            host_GF_Efshift = self.inputs.remote_data_Efshift
+        else:
+            host_GF_Efshift = None
 
         options = {"max_wallclock_seconds": self.ctx.max_wallclock_seconds,
                    "resources": self.ctx.resources,
@@ -674,19 +681,20 @@ class kkr_imp_sub_wc(WorkChain):
             emin = GF_out_params.get_dict().get('energy_contour_group').get('emin')
             # then use this value to get rid of all core states that are lower than emin (return the same input potential if no states have been removed
             imp_pot = kick_out_corestates_wf(imp_pot, Float(emin))
+            self.ctx.sfd_pot_to_clean.append(imp_pot)
             if 'impurity_info' in self.inputs:
                 self.report('INFO: using impurity_info node as input for kkrimp calculation')
                 imp_info = self.inputs.impurity_info
                 label = 'KKRimp calculation step {} (IMIX={}, Zimp: {})'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme, imp_info.get_dict().get('Zimp'))
                 description = 'KKRimp calculation of step {}, using mixing scheme {}'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme)
                 inputs = get_inputs_kkrimp(code, options, label, description, params,
-                                           not self.ctx.use_mpi, imp_info=imp_info, host_GF=host_GF, imp_pot=imp_pot)
+                                           not self.ctx.use_mpi, imp_info=imp_info, host_GF=host_GF, imp_pot=imp_pot, host_GF_Efshift=host_GF_Efshift)
             else:
                 self.report('INFO: getting impurity_info node from previous GF calculation')
                 label = 'KKRimp calculation step {} (IMIX={}, GF_remote: {})'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme, host_GF.pk)
                 description = 'KKRimp calculation of step {}, using mixing scheme {}'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme)
                 inputs = get_inputs_kkrimp(code, options, label, description, params,
-                                           not self.ctx.use_mpi, host_GF=host_GF, imp_pot=imp_pot)
+                                           not self.ctx.use_mpi, host_GF=host_GF, imp_pot=imp_pot, host_GF_Efshift=host_GF_Efshift)
         elif last_remote is not None:
             # fix to get Zimp properly
             if 'impurity_info' in self.inputs:
@@ -695,13 +703,13 @@ class kkr_imp_sub_wc(WorkChain):
                 label = 'KKRimp calculation step {} (IMIX={}, Zimp: {})'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme, imp_info.get_dict().get('Zimp'))
                 description = 'KKRimp calculation of step {}, using mixing scheme {}'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme)
                 inputs = get_inputs_kkrimp(code, options, label, description, params,
-                                           not self.ctx.use_mpi, imp_info=imp_info, host_GF=host_GF, kkrimp_remote=last_remote)
+                                           not self.ctx.use_mpi, imp_info=imp_info, host_GF=host_GF, kkrimp_remote=last_remote, host_GF_Efshift=host_GF_Efshift)
             else:
                 self.report('INFO: using RemoteData from previous kkrimp calculation as input')
                 label = 'KKRimp calculation step {} (IMIX={}, Zimp: {})'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme, None)
                 description = 'KKRimp calculation of step {}, using mixing scheme {}'.format(self.ctx.loop_count, self.ctx.last_mixing_scheme)
                 inputs = get_inputs_kkrimp(code, options, label, description, params,
-                                           not self.ctx.use_mpi, host_GF=host_GF, kkrimp_remote=last_remote)
+                                           not self.ctx.use_mpi, host_GF=host_GF, kkrimp_remote=last_remote, host_GF_Efshift=host_GF_Efshift)
 
         # run the KKR calculation
         self.report('INFO: doing calculation')
@@ -722,6 +730,7 @@ class kkr_imp_sub_wc(WorkChain):
         # check calculation state
         if not self.ctx.last_calc.is_finished_ok:
             self.ctx.kkrimp_step_success = False
+            self.report("ERROR: last calc not finished_ok")
             return self.exit_codes.ERROR_LAST_CALC_NOT_FINISHED_OK
 
         self.report("INFO: kkrimp_step_success: {}".format(self.ctx.kkrimp_step_success))
@@ -745,7 +754,9 @@ class kkr_imp_sub_wc(WorkChain):
                 # take potfile directly from output
                 with retrieved_folder.open(KkrimpCalculation._OUT_POTENTIAL, 'rb') as pot_file:
                     self.ctx.last_pot = SinglefileData(file=pot_file)
+            self.ctx.sfd_pot_to_clean.append(self.ctx.last_pot)
         except:
+            self.report("ERROR: no output potential found")
             return self.exit_codes.ERROR_NO_OUTPUT_POT_FROM_LAST_CALC
 
         # extract convergence info about rms etc. (used to determine convergence behavior)
@@ -973,7 +984,9 @@ class kkr_imp_sub_wc(WorkChain):
         outputnode_t.store()
 
         self.out('workflow_info', outputnode_t)
-        self.out('host_imp_pot', self.ctx.last_pot)
+        # store out_potential as SingleFileData only if this was no DOS run
+        if not self.ctx.dos_run:
+            self.out('host_imp_pot', self.ctx.last_pot)
 
         # print results table for overview
         # table layout:
@@ -1000,6 +1013,17 @@ class kkr_imp_sub_wc(WorkChain):
             """
         self.report(message)
 
+     
+        if self.ctx.successful:
+            self.report("INFO: clean output of calcs")
+            remove_out_pot_impcalcs(self.ctx.successful, all_pks)
+            self.report("INFO: clean up raw_input folders")
+            clean_raw_input(self.ctx.successful, all_pks)
+
+        # clean intermediate single file data which are not needed after successful run or after DOS run
+        if self.ctx.successful or self.ctx.dos_run:
+            self.final_cleanup()
+
         self.report("INFO: done with kkr_scf workflow!\n")
 
 
@@ -1007,3 +1031,123 @@ class kkr_imp_sub_wc(WorkChain):
         """Capture errors raised in validate_input"""
         if self.ctx.exit_code is not None:
             return self.ctx.exit_code
+
+    def final_cleanup(self):
+        uuid_last_calc = self.ctx.last_pot.uuid
+        if not self.ctx.dos_run:
+            sfds_to_clean = [i for i in self.ctx.sfd_pot_to_clean if i.uuid!=uuid_last_calc]
+        else:
+            # in case of DOS run we can also clean the last output sfd file since this is never used
+            sfds_to_clean = self.ctx.sfd_pot_to_clean
+        # now clean all sfd files that are not needed anymore
+        for sfd_to_clean in sfds_to_clean:
+            clean_sfd(sfd_to_clean)
+
+
+def remove_out_pot_impcalcs(successful, pks_all_calcs, dry_run=False):
+    """
+    Remove out_potential file from all but the last KKRimp calculation if workflow was successful
+    Usage:
+        imp_wf = load_node(266885) # maybe start with outer workflow
+        pk_imp_scf = imp_wf.outputs.workflow_info['used_subworkflows'].get('kkr_imp_sub')
+        imp_scf_wf = load_node(pk_imp_scf) # this is now the imp scf sub workflow
+        successful = imp_scf_wf.outputs.workflow_info['successful']
+        pks_all_calcs = imp_scf_wf.outputs.workflow_info['pks_all_calcs']
+    """
+    import tarfile, os
+    from aiida.orm import load_node
+    from aiida.common.folders import SandboxFolder
+    from aiida_kkr.calculations import KkrimpCalculation
+    
+    if dry_run:
+        print('test', successful, len(pks_all_calcs))
+
+    # name of tarfile
+    tfname = KkrimpCalculation._FILENAME_TAR
+    
+    # cleanup only if calculation was successful
+    if successful and len(pks_all_calcs)>1:
+        # remove out_potential for calculations
+        # note that also last calc can be cleaned since output potential is stored in single file data
+        pks_for_cleanup = pks_all_calcs[:]
+
+        # loop over all calculations
+        for pk in pks_for_cleanup:
+            if dry_run:
+                print('pk_for_cleanup:', pk)
+            # get getreived folder of calc
+            calc = load_node(pk)
+            ret = calc.outputs.retrieved
+
+            # open tarfile if present
+            if tfname in ret.list_object_names():
+                delete_and_retar = False
+                with ret.open(tfname) as tf:
+                    tf_abspath = tf.name
+
+                # create Sandbox folder which is used to temporarily extract output_all.tar.gz
+                tmpfolder = SandboxFolder()
+                tmpfolder_path = tmpfolder.abspath
+                with tarfile.open(tf_abspath) as tf:
+                    tar_filenames = [ifile.name for ifile in tf.getmembers()]
+                    # check if out_potential is in tarfile
+                    if KkrimpCalculation._OUT_POTENTIAL in tar_filenames:
+                        tf.extractall(tmpfolder_path)
+                        delete_and_retar = True
+
+                if delete_and_retar and not dry_run:
+                    # delete out_potential
+                    os.remove(os.path.join(tmpfolder_path, KkrimpCalculation._OUT_POTENTIAL))
+                    with tarfile.open(tf_abspath, 'w:gz') as tf:
+                        # remove out_potential from list of files
+                        tar_filenames = [i for i in tar_filenames if i!=KkrimpCalculation._OUT_POTENTIAL]
+                        for f in tar_filenames:
+                            # create new tarfile without out_potential file
+                            fabs = os.path.join(tmpfolder_path, f)
+                            tf.add(fabs, arcname=os.path.basename(fabs))
+                elif dry_run:
+                    print('dry run:')
+                    print('delete and retar?', delete_and_retar)
+                    print('tmpfolder_path', tmpfolder_path)
+
+                # clean up temporary Sandbox folder
+                if not dry_run:
+                    tmpfolder.erase()
+
+def clean_raw_input(successful, pks_calcs, dry_run=False):
+    """
+    Clean raw_input directories that contain copies of shapefun and potential files
+    This however breaks provenance (strictly speaking) and therefore should only be done 
+    for the calculations of a successfully finished workflow (see email on mailing list from 25.11.2019).
+    """
+    from aiida.orm import load_node
+    from aiida_kkr.calculations import KkrimpCalculation
+    if successful:
+        for pk in pks_calcs:
+            node = load_node(pk)
+            # clean only nodes that are KkrimpCalculations
+            if node.process_class==KkrimpCalculation:
+                raw_input_folder = node._raw_input_folder
+                # clean potential and shapefun files
+                for filename in [KkrimpCalculation._POTENTIAL, KkrimpCalculation._SHAPEFUN]:
+                    if filename in raw_input_folder.get_content_list():
+                        if dry_run:
+                            print('clean {}'.format(filename))
+                        else:
+                            raw_input_folder.remove_path(filename)
+    elif dry_run:
+        print('no raw_inputs to clean')
+
+
+
+def clean_sfd(sfd_to_clean, nkeep=30):
+    with sfd_to_clean.open(sfd_to_clean.filename) as f:
+        txt = f.readlines()
+    # remove all lines after nkeep lines
+    txt2 = txt[:nkeep]
+    # add note to end of file
+    txt2+= [u'WARNING: REST OF FILE WAS CLEANED SO SAVE SPACE!!!\n']
+    # overwrite file
+    with sfd_to_clean.open(sfd_to_clean.filename, 'w') as fnew:
+        fnew.writelines(txt2)
+
